//    RW Shell
// ==============
// Robowaifu Shell, and General Inboard Systems Library
// -Part of the RW Foundations suite of libraries

// Filename: ./include/shell/shell.hpp
// -Declarations of the baseline RW Shell library

#pragma once

#include "incls_all.hpp"

namespace rw::shell {

//------------------------------------------------------------------------------
/**
 */
class Shell_b : public RW {
 public:
  Shell_b();                                    // default ctor
  explicit Shell_b(std::string const& handle);  // tag-parm'd ctor
};

//------------------------------------------------------------------------------
/**
 */
class Shell : public Shell_b {
 public:
  Shell();                                    // default ctor
  explicit Shell(std::string const& handle);  // tag-parm'd ctor

  // shell & power care
  void dock_me() {}
  void charge_me() {}
  void sleep_me() {}
  void wake_me() {}
  void store_me() {}

  inline auto dollhouse() const -> Dollhouse& { return dollhouse_; }
  inline auto electro() const -> Electro& { return electro_; }
  inline auto gears() const -> Gears& { return gears_; }

 private:
  unique_ptr<Dollhouse_b> _dollhouse_{
      make_unique<Dollhouse_b>("shell_dollhouse")};
  unique_ptr<Electro_b> _electro_{make_unique<Electro_b>("shell_electro")};
  unique_ptr<Gears_b>   _gears_{make_unique<Gears_b>("shell_gears")};

  Dollhouse& dollhouse_{reference_to<Dollhouse>(_dollhouse_.get())};
  Electro&   electro_{reference_to<Electro>(_electro_.get())};
  Gears&     gears_{reference_to<Gears>(_gears_.get())};
};

}  // namespace rw::shell

//    --=^=--

// "______"

// rw_shell_lib.v220507
// Copyright (2022)
// License MIT (Expat) https://opensource.org/licenses/MIT
